==> gradle-9.4.0-20251215022449+0000-src.zip: gradle-9.4.0-20251215022449+0000/platforms/core-configuration/kotlin-dsl/src/integTest/kotlin/org/gradle/kotlin/dsl/dcl/OptInDclAccessorsIntegrationTest.kt <==
/*
 * Copyright 2025 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.kotlin.dsl.dcl

import org.gradle.api.internal.plugins.BindsProjectType
import org.gradle.api.internal.plugins.BuildModel
import org.gradle.api.internal.plugins.Definition
import org.gradle.api.internal.plugins.ProjectTypeBinding
import org.gradle.api.internal.plugins.ProjectTypeBindingBuilder
import org.gradle.kotlin.dsl.accessors.DCL_ENABLED_PROPERTY_NAME
import org.gradle.kotlin.dsl.fixtures.AbstractKotlinIntegrationTest
import kotlin.test.Test

class OptInDclAccessorsIntegrationTest : AbstractKotlinIntegrationTest() {

    @Test
    fun `propagates opt-in annotations to DCL accessors`() {
        withEcosystemAndPluginBuildInBuildLogic()
        withEcosystemPluginRegisteringMyPlugin()
        withDeclarativePluginWithOptInRequirements()
        withDclEnabledInGradleProperties()

        file("settings.gradle.kts").appendText(
            "\n" + """
            import com.example.SomeExperimentalApi

            defaults {
                @OptIn(SomeExperimentalApi::class)
                myProjectType { }
            }
        """.trimIndent()
        )

        withBuildScript(
            """
            import com.example.SomeExperimentalApi

            @OptIn(SomeExperimentalApi::class)
            myProjectType {
                myElements {
                    myElement("foo") { }
                }
            }
        """.trimIndent()
        )

        //TODO: DCL cannot expose the project type accessors to a KTS precompiled script plugin yet, as it needs an ecosystem plugin to
        // register the models; so far we cannot test that the sources generated for the DCL accessors are valid by actually building against them.
        build("kotlinDslAccessorsReport").apply {
            assertNotOutput("w:")
            assertOutputContains(
                """
                |    @com.example.SomeExperimentalApi
                |    fun org.gradle.api.NamedDomainObjectContainer<com.example.MyElement>.`myElement`
                """.trimMargin()
            )

            assertOutputContains(
                """
                |    @com.example.SomeExperimentalApi
                |    fun org.gradle.api.Project.`myProjectType`(configure: Action<in com.example.MyExtension>) {
                """.trimMargin()
            )
        }
    }

    private fun withEcosystemPluginRegisteringMyPlugin() {
        withFile(
            "build-logic/src/main/kotlin/MyEcosystemPlugin.kt", """
                package com.example

                import org.gradle.api.Plugin
                import org.gradle.api.initialization.Settings
                import org.gradle.api.internal.plugins.software.RegistersSoftwareTypes

                @RegistersSoftwareTypes(MyPlugin::class)
                class MyEcosystemPlugin : Plugin<Settings> {
                    override fun apply(settings: Settings) = Unit
                }
            """.trimIndent()
        )
    }

    private fun withDclEnabledInGradleProperties() =
        withFile("gradle.properties").appendText("\n$DCL_ENABLED_PROPERTY_NAME=true\n")

    private fun withEcosystemAndPluginBuildInBuildLogic() {
        withFile(
            "build-logic/build.gradle.kts", """
                import org.jetbrains.kotlin.gradle.dsl.JvmTarget

                plugins {
                    id("java-gradle-plugin")
                    `kotlin-dsl`
                }

                kotlin {
                    compilerOptions {
                        jvmTarget = JvmTarget.JVM_1_8
                    }
                }

                tasks.compileJava {
                    targetCompatibility = "1.8"
                }

                repositories {
                    mavenCentral()
                }

                gradlePlugin {
                    plugins {
                        create("myPlugin") {
                            id = "com.example.myPlugin"
                            implementationClass = "com.example.MyPlugin"
                        }
                        create("myEcosystemPlugin") {
                            id = "com.example.myEcosystemPlugin"
                            implementationClass = "com.example.MyEcosystemPlugin"
                        }
                    }
                }
            """.trimIndent()
        )

        withFile(
            "settings.gradle.kts", """
            pluginManagement {
                includeBuild("build-logic")
            }

            plugins {
                id("com.example.myEcosystemPlugin")
            }
        """.trimIndent()
        )
    }

    private fun withDeclarativePluginWithOptInRequirements() {
        withEcosystemAndPluginBuildInBuildLogic()

        withEcosystemPluginRegisteringMyPlugin()

        withFile(
            "build-logic/src/main/kotlin/MyPlugin.kt", """
                package com.example

                import org.gradle.api.Plugin
                import org.gradle.api.Project
                import org.gradle.api.Named
                import org.gradle.api.NamedDomainObjectContainer
                import javax.inject.Inject
                import ${BindsProjectType::class.java.name}
                import ${ProjectTypeBinding::class.java.name}
                import ${ProjectTypeBindingBuilder::class.java.name}
                import ${Definition::class.java.name}
                import ${BuildModel::class.java.name}
                import org.gradle.api.internal.plugins.features.dsl.bindProjectType

                @RequiresOptIn("Some Experimental API", RequiresOptIn.Level.ERROR)
                annotation class SomeExperimentalApi

                @OptIn(SomeExperimentalApi::class)
                @Suppress("deprecation")
                @${BindsProjectType::class.java.simpleName}(MyPlugin.Binding::class)
                abstract class MyPlugin @Inject constructor(private val project: Project) : Plugin<Project> {
                    class Binding : ${ProjectTypeBinding::class.java.simpleName} {
                        override fun bind(builder: ${ProjectTypeBindingBuilder::class.java.simpleName}) {
                            builder.bindProjectType("myProjectType") { definition: MyExtension, model -> }
                        }
                    }

                    override fun apply(project: Project) = Unit
                }

                @SomeExperimentalApi
                @Suppress("deprecation")
                interface MyExtension : ${Definition::class.java.simpleName}<Model> {
                    val myElements: NamedDomainObjectContainer<MyElement>
                }

                interface Model : ${BuildModel::class.java.simpleName} { }

                @SomeExperimentalApi
                abstract class MyElement(val elementName: String) : Named {
                    override fun getName() = elementName
                }
                """.trimIndent()
        )
    }
}

==> gradle-9.5.0-20260221010500+0000-src.zip: gradle-9.5.0-20260221010500+0000/platforms/core-configuration/kotlin-dsl/src/integTest/kotlin/org/gradle/kotlin/dsl/dcl/OptInDclAccessorsIntegrationTest.kt <==
/*
 * Copyright 2025 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.kotlin.dsl.dcl

import org.gradle.features.annotations.BindsProjectType
import org.gradle.features.annotations.RegistersProjectFeatures
import org.gradle.features.binding.BuildModel
import org.gradle.features.binding.Definition
import org.gradle.features.binding.ProjectTypeBinding
import org.gradle.features.binding.ProjectTypeBindingBuilder
import org.gradle.kotlin.dsl.accessors.DCL_ENABLED_PROPERTY_NAME
import org.gradle.kotlin.dsl.fixtures.AbstractKotlinIntegrationTest
import kotlin.test.Test

class OptInDclAccessorsIntegrationTest : AbstractKotlinIntegrationTest() {

    @Test
    fun `propagates opt-in annotations to DCL accessors`() {
        withEcosystemAndPluginBuildInBuildLogic()
        withEcosystemPluginRegisteringMyPlugin()
        withDeclarativePluginWithOptInRequirements()
        withDclEnabledInGradleProperties()

        file("settings.gradle.kts").appendText(
            "\n" + """
            import com.example.SomeExperimentalApi

            defaults {
                @OptIn(SomeExperimentalApi::class)
                myProjectType { }
            }
        """.trimIndent()
        )

        withBuildScript(
            """
            import com.example.SomeExperimentalApi

            @OptIn(SomeExperimentalApi::class)
            myProjectType {
                myElements {
                    myElement("foo") { }
                }
            }
        """.trimIndent()
        )

        //TODO: DCL cannot expose the project type accessors to a KTS precompiled script plugin yet, as it needs an ecosystem plugin to
        // register the models; so far we cannot test that the sources generated for the DCL accessors are valid by actually building against them.
        build("kotlinDslAccessorsReport").apply {
            assertNotOutput("w:")
            assertOutputContains(
                """
                |    @com.example.SomeExperimentalApi
                |    fun org.gradle.api.NamedDomainObjectContainer<com.example.MyElement>.`myElement`
                """.trimMargin()
            )

            assertOutputContains(
                """
                |    @com.example.SomeExperimentalApi
                |    fun org.gradle.api.Project.`myProjectType`(configure: Action<in com.example.MyExtension>) {
                """.trimMargin()
            )
        }
    }

    private fun withEcosystemPluginRegisteringMyPlugin() {
        withFile(
            "build-logic/src/main/kotlin/MyEcosystemPlugin.kt", """
                package com.example

                import org.gradle.api.Plugin
                import org.gradle.api.initialization.Settings
                import ${RegistersProjectFeatures::class.qualifiedName}

                @${RegistersProjectFeatures::class.simpleName}(MyPlugin::class)
                class MyEcosystemPlugin : Plugin<Settings> {
                    override fun apply(settings: Settings) = Unit
                }
            """.trimIndent()
        )
    }

    private fun withDclEnabledInGradleProperties() =
        withFile("gradle.properties").appendText("\n$DCL_ENABLED_PROPERTY_NAME=true\n")

    private fun withEcosystemAndPluginBuildInBuildLogic() {
        withFile(
            "build-logic/build.gradle.kts", """
                import org.jetbrains.kotlin.gradle.dsl.JvmTarget

                plugins {
                    id("java-gradle-plugin")
                    `kotlin-dsl`
                }

                kotlin {
                    compilerOptions {
                        jvmTarget = JvmTarget.JVM_1_8
                    }
                }

                tasks.compileJava {
                    targetCompatibility = "1.8"
                }

                repositories {
                    mavenCentral()
                }

                gradlePlugin {
                    plugins {
                        create("myPlugin") {
                            id = "com.example.myPlugin"
                            implementationClass = "com.example.MyPlugin"
                        }
                        create("myEcosystemPlugin") {
                            id = "com.example.myEcosystemPlugin"
                            implementationClass = "com.example.MyEcosystemPlugin"
                        }
                    }
                }
            """.trimIndent()
        )

        withFile(
            "settings.gradle.kts", """
            pluginManagement {
                includeBuild("build-logic")
            }

            plugins {
                id("com.example.myEcosystemPlugin")
            }
        """.trimIndent()
        )
    }

    private fun withDeclarativePluginWithOptInRequirements() {
        withEcosystemAndPluginBuildInBuildLogic()

        withEcosystemPluginRegisteringMyPlugin()

        withFile(
            "build-logic/src/main/kotlin/MyPlugin.kt", """
                package com.example

                import org.gradle.api.Plugin
                import org.gradle.api.Project
                import org.gradle.api.Named
                import org.gradle.api.NamedDomainObjectContainer
                import javax.inject.Inject
                import ${BindsProjectType::class.java.name}
                import ${ProjectTypeBinding::class.java.name}
                import ${ProjectTypeBindingBuilder::class.java.name}
                import ${Definition::class.java.name}
                import ${BuildModel::class.java.name}
                import org.gradle.features.dsl.bindProjectType

                @RequiresOptIn("Some Experimental API", RequiresOptIn.Level.ERROR)
                annotation class SomeExperimentalApi

                @OptIn(SomeExperimentalApi::class)
                @Suppress("deprecation")
                @${BindsProjectType::class.java.simpleName}(MyPlugin.Binding::class)
                abstract class MyPlugin @Inject constructor(private val project: Project) : Plugin<Project> {
                    class Binding : ${ProjectTypeBinding::class.java.simpleName} {
                        override fun bind(builder: ${ProjectTypeBindingBuilder::class.java.simpleName}) {
                            builder.bindProjectType("myProjectType") { definition: MyExtension, model -> }
                        }
                    }

                    override fun apply(project: Project) = Unit
                }

                @SomeExperimentalApi
                @Suppress("deprecation")
                interface MyExtension : ${Definition::class.java.simpleName}<Model> {
                    val myElements: NamedDomainObjectContainer<MyElement>
                }

                interface Model : ${BuildModel::class.java.simpleName} { }

                @SomeExperimentalApi
                abstract class MyElement(val elementName: String) : Named {
                    override fun getName() = elementName
                }
                """.trimIndent()
        )
    }
}
